@@ -25,36 +25,33 @@
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+   You can contact the author at :
+   - LZ4 homepage : http://fastcompression.blogspot.com/p/lz4.html
+   - LZ4 source repository : http://code.google.com/p/lz4/
 */
 
 //**************************************
 // Tuning parameters
 //**************************************
-// COMPRESSIONLEVEL :
-// Increasing this value improves compression ratio
-// Lowering this value reduces memory usage
-// Reduced memory usage typically improves speed, due to cache effect (ex : L1 32KB for Intel, L1 64KB for AMD)
-// Memory usage formula : N->2^(N+2) Bytes (examples : 12 -> 16KB ; 17 -> 512KB)
-#define COMPRESSIONLEVEL 12
+// MEMORY_USAGE :
+// Memory usage formula : N->2^N Bytes (examples : 10 -> 1KB; 12 -> 4KB ; 16 -> 64KB; 20 -> 1MB; etc.)
+// Increasing memory usage improves compression ratio
+// Reduced memory usage can improve speed, due to cache effect
+// Default value is 14, for 16KB, which nicely fits into Intel x86 L1 cache
+#define MEMORY_USAGE 14
 
-// NOTCOMPRESSIBLE_CONFIRMATION :
+// NOTCOMPRESSIBLE_DETECTIONLEVEL :
 // Decreasing this value will make the algorithm skip faster data segments considered "incompressible"
 // This may decrease compression ratio dramatically, but will be faster on incompressible data
 // Increasing this value will make the algorithm search more before declaring a segment "incompressible"
 // This could improve compression a bit, but will be slower on incompressible data
 // The default value (6) is recommended
-#define NOTCOMPRESSIBLE_CONFIRMATION 6
-
-// LZ4_COMPRESSMIN :
-// Compression function will *fail* if it is not successful at compressing input by at least LZ4_COMPRESSMIN bytes
-// Since the compression function stops working prematurely, it results in a speed gain
-// The output however is unusable. Compression function result will be zero.
-// Default : 0 = disabled
-#define LZ4_COMPRESSMIN 0
+#define NOTCOMPRESSIBLE_DETECTIONLEVEL 6
 
 // BIG_ENDIAN_NATIVE_BUT_INCOMPATIBLE :
-// This will provide a boost to performance for big endian cpu, but the resulting compressed stream will be incompatible with little-endian CPU.
-// You can set this option to 1 in situations where data will stay within closed environment
+// This will provide a small boost to performance for big endian cpu, but the resulting compressed stream will be incompatible with little-endian CPU.
+// You can set this option to 1 in situations where data will remain within closed environment
 // This option is useless on Little_Endian CPU (such as x86)
 //#define BIG_ENDIAN_NATIVE_BUT_INCOMPATIBLE 1
 
@@ -65,14 +62,15 @@
 //**************************************
 // 32 or 64 bits ?
 #if (defined(__x86_64__) || defined(__x86_64) || defined(__amd64__) || defined(__amd64) || defined(__ppc64__) || defined(_WIN64) || defined(__LP64__) || defined(_LP64) )   // Detects 64 bits mode
-#define LZ4_ARCH64 1
+#  define LZ4_ARCH64 1
 #else
-#define LZ4_ARCH64 0
+#  define LZ4_ARCH64 0
 #endif
 
-// Little Endian or Big Endian ? 
+// Little Endian or Big Endian ?
+// Note : overwrite the below #define if you know your architecture endianess
 #if (defined(__BIG_ENDIAN__) || defined(__BIG_ENDIAN) || defined(_BIG_ENDIAN) || defined(_ARCH_PPC) || defined(__PPC__) || defined(__PPC) || defined(PPC) || defined(__powerpc__) || defined(__powerpc) || defined(powerpc) || ((defined(__BYTE_ORDER__)&&(__BYTE_ORDER__ == __ORDER_BIG_ENDIAN__))) )
-#define LZ4_BIG_ENDIAN 1
+#  define LZ4_BIG_ENDIAN 1
 #else
 // Little Endian assumed. PDP Endian and other very rare endian format are unsupported.
 #endif
@@ -81,12 +79,13 @@
 // For others CPU, the compiler will be more cautious, and insert extra code to ensure aligned access is respected
 // If you know your target CPU supports unaligned memory access, you may want to force this option manually to improve performance
 #if defined(__ARM_FEATURE_UNALIGNED)
-#define LZ4_FORCE_UNALIGNED_ACCESS 1
+#  define LZ4_FORCE_UNALIGNED_ACCESS 1
 #endif
 
-// Uncomment this parameter if your target system or compiler does not support hardware bit count
-//#define LZ4_FORCE_SW_BITCOUNT
-
+// Define this parameter if your target system or compiler does not support hardware bit count
+#if defined(_MSC_VER) && defined(_WIN32_WCE)            // Visual Studio for Windows CE does not support Hardware bit count
+#  define LZ4_FORCE_SW_BITCOUNT
+#endif
 
 
 //**************************************
@@ -95,26 +94,32 @@
 #if __STDC_VERSION__ >= 199901L // C99
 /* "restrict" is a known keyword */
 #else
-#define restrict // Disable restrict
+#  define restrict // Disable restrict
 #endif
 
 #define GCC_VERSION (__GNUC__ * 100 + __GNUC_MINOR__)
 
 #ifdef _MSC_VER  // Visual Studio
-#define inline __forceinline // Visual is not C99, but supports some kind of inline
-#include <intrin.h>          // _BitScanForward
+#  define inline __forceinline           // Visual is not C99, but supports some kind of inline
+#  if LZ4_ARCH64	// 64-bit
+#    pragma intrinsic(_BitScanForward64) // For Visual 2005
+#    pragma intrinsic(_BitScanReverse64) // For Visual 2005
+#  else
+#    pragma intrinsic(_BitScanForward)   // For Visual 2005
+#    pragma intrinsic(_BitScanReverse)   // For Visual 2005
+#  endif
 #endif
 
-#ifdef _MSC_VER 
-#define lz4_bswap16(x) _byteswap_ushort(x)
+#ifdef _MSC_VER
+#  define lz4_bswap16(x) _byteswap_ushort(x)
 #else
-#define lz4_bswap16(x) ((unsigned short int) ((((x) >> 8) & 0xffu) | (((x) & 0xffu) << 8)))
+#  define lz4_bswap16(x) ((unsigned short int) ((((x) >> 8) & 0xffu) | (((x) & 0xffu) << 8)))
 #endif
 
 #if (GCC_VERSION >= 302) || (__INTEL_COMPILER >= 800) || defined(__clang__)
-# define expect(expr,value)    (__builtin_expect ((expr),(value)) )
+#  define expect(expr,value)    (__builtin_expect ((expr),(value)) )
 #else
-# define expect(expr,value)    (expr)
+#  define expect(expr,value)    (expr)
 #endif
 
 #define likely(expr)     expect((expr) != 0, 1)
@@ -133,22 +138,22 @@
 // Basic Types
 //**************************************
 #if defined(_MSC_VER)    // Visual Studio does not support 'stdint' natively
-#define BYTE	unsigned __int8
-#define U16		unsigned __int16
-#define U32		unsigned __int32
-#define S32		__int32
-#define U64		unsigned __int64
+#  define BYTE	unsigned __int8
+#  define U16		unsigned __int16
+#  define U32		unsigned __int32
+#  define S32		__int32
+#  define U64		unsigned __int64
 #else
-#include <stdint.h>
-#define BYTE	uint8_t
-#define U16		uint16_t
-#define U32		uint32_t
-#define S32		int32_t
-#define U64		uint64_t
+#  include <stdint.h>
+#  define BYTE	uint8_t
+#  define U16		uint16_t
+#  define U32		uint32_t
+#  define S32		int32_t
+#  define U64		uint64_t
 #endif
 
 #ifndef LZ4_FORCE_UNALIGNED_ACCESS
-#pragma pack(push, 1) 
+#  pragma pack(push, 1)
 #endif
 
 typedef struct _U16_S { U16 v; } U16_S;
@@ -156,7 +161,7 @@ typedef struct _U32_S { U32 v; } U32_S;
 typedef struct _U64_S { U64 v; } U64_S;
 
 #ifndef LZ4_FORCE_UNALIGNED_ACCESS
-#pragma pack(pop) 
+#  pragma pack(pop)
 #endif
 
 #define A64(x) (((U64_S *)(x))->v)
@@ -169,11 +174,11 @@ typedef struct _U64_S { U64 v; } U64_S;
 //**************************************
 #define MINMATCH 4
 
-#define HASH_LOG COMPRESSIONLEVEL
+#define HASH_LOG (MEMORY_USAGE-2)
 #define HASHTABLESIZE (1 << HASH_LOG)
 #define HASH_MASK (HASHTABLESIZE - 1)
 
-#define SKIPSTRENGTH (NOTCOMPRESSIBLE_CONFIRMATION>2?NOTCOMPRESSIBLE_CONFIRMATION:2)
+#define SKIPSTRENGTH (NOTCOMPRESSIBLE_DETECTIONLEVEL>2?NOTCOMPRESSIBLE_DETECTIONLEVEL:2)
 #define STACKLIMIT 13
 #define HEAPMODE (HASH_LOG>STACKLIMIT)  // Defines if memory is allocated into the stack (local variable), or into the heap (malloc()).
 #define COPYLENGTH 8
@@ -194,31 +199,31 @@ typedef struct _U64_S { U64 v; } U64_S;
 // Architecture-specific macros
 //**************************************
 #if LZ4_ARCH64	// 64-bit
-#define STEPSIZE 8
-#define UARCH U64
-#define AARCH A64
-#define LZ4_COPYSTEP(s,d)		A64(d) = A64(s); d+=8; s+=8;
-#define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d)
-#define LZ4_SECURECOPY(s,d,e)	if (d<e) LZ4_WILDCOPY(s,d,e)
-#define HTYPE U32
-#define INITBASE(base)			const BYTE* const base = ip
+#  define STEPSIZE 8
+#  define UARCH U64
+#  define AARCH A64
+#  define LZ4_COPYSTEP(s,d)		A64(d) = A64(s); d+=8; s+=8;
+#  define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d)
+#  define LZ4_SECURECOPY(s,d,e)	if (d<e) LZ4_WILDCOPY(s,d,e)
+#  define HTYPE U32
+#  define INITBASE(base)			const BYTE* const base = ip
 #else		// 32-bit
-#define STEPSIZE 4
-#define UARCH U32
-#define AARCH A32
-#define LZ4_COPYSTEP(s,d)		A32(d) = A32(s); d+=4; s+=4;
-#define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d); LZ4_COPYSTEP(s,d);
-#define LZ4_SECURECOPY			LZ4_WILDCOPY
-#define HTYPE const BYTE*
-#define INITBASE(base)			const int base = 0
+#  define STEPSIZE 4
+#  define UARCH U32
+#  define AARCH A32
+#  define LZ4_COPYSTEP(s,d)		A32(d) = A32(s); d+=4; s+=4;
+#  define LZ4_COPYPACKET(s,d)		LZ4_COPYSTEP(s,d); LZ4_COPYSTEP(s,d);
+#  define LZ4_SECURECOPY			LZ4_WILDCOPY
+#  define HTYPE const BYTE*
+#  define INITBASE(base)			const int base = 0
 #endif
 
 #if (defined(LZ4_BIG_ENDIAN) && !defined(BIG_ENDIAN_NATIVE_BUT_INCOMPATIBLE))
-#define LZ4_READ_LITTLEENDIAN_16(d,s,p) { U16 v = A16(p); v = lz4_bswap16(v); d = (s) - v; }
-#define LZ4_WRITE_LITTLEENDIAN_16(p,i)  { U16 v = (U16)(i); v = lz4_bswap16(v); A16(p) = v; p+=2; }
+#  define LZ4_READ_LITTLEENDIAN_16(d,s,p) { U16 v = A16(p); v = lz4_bswap16(v); d = (s) - v; }
+#  define LZ4_WRITE_LITTLEENDIAN_16(p,i)  { U16 v = (U16)(i); v = lz4_bswap16(v); A16(p) = v; p+=2; }
 #else		// Little Endian
-#define LZ4_READ_LITTLEENDIAN_16(d,s,p) { d = (s) - A16(p); }
-#define LZ4_WRITE_LITTLEENDIAN_16(p,v)  { A16(p) = v; p+=2; }
+#  define LZ4_READ_LITTLEENDIAN_16(d,s,p) { d = (s) - A16(p); }
+#  define LZ4_WRITE_LITTLEENDIAN_16(p,v)  { A16(p) = v; p+=2; }
 #endif
 
 
@@ -245,7 +250,7 @@ struct refTables
 //****************************
 #if LZ4_ARCH64
 
-inline static int LZ4_NbCommonBytes (register U64 val)
+static inline int LZ4_NbCommonBytes (register U64 val)
 {
 #if defined(LZ4_BIG_ENDIAN)
     #if defined(_MSC_VER) && !defined(LZ4_FORCE_SW_BITCOUNT)
@@ -253,7 +258,7 @@ inline static int LZ4_NbCommonBytes (register U64 val)
     _BitScanReverse64( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_clzll(val) >> 3); 
+    return (__builtin_clzll(val) >> 3);
     #else
 	int r;
 	if (!(val>>32)) { r=4; } else { r=0; val>>=32; }
@@ -267,7 +272,7 @@ inline static int LZ4_NbCommonBytes (register U64 val)
     _BitScanForward64( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_ctzll(val) >> 3); 
+    return (__builtin_ctzll(val) >> 3);
     #else
 	static const int DeBruijnBytePos[64] = { 0, 0, 0, 0, 0, 1, 1, 2, 0, 3, 1, 3, 1, 4, 2, 7, 0, 2, 3, 6, 1, 5, 3, 5, 1, 3, 4, 4, 2, 5, 6, 7, 7, 0, 1, 2, 3, 3, 4, 6, 2, 6, 5, 5, 3, 4, 5, 6, 7, 1, 2, 4, 6, 4, 4, 5, 7, 2, 6, 5, 7, 6, 7, 7 };
 	return DeBruijnBytePos[((U64)((val & -val) * 0x0218A392CDABBD3F)) >> 58];
@@ -277,7 +282,7 @@ inline static int LZ4_NbCommonBytes (register U64 val)
 
 #else
 
-inline static int LZ4_NbCommonBytes (register U32 val)
+static inline int LZ4_NbCommonBytes (register U32 val)
 {
 #if defined(LZ4_BIG_ENDIAN)
     #if defined(_MSC_VER) && !defined(LZ4_FORCE_SW_BITCOUNT)
@@ -285,7 +290,7 @@ inline static int LZ4_NbCommonBytes (register U32 val)
     _BitScanReverse( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_clz(val) >> 3); 
+    return (__builtin_clz(val) >> 3);
     #else
 	int r;
 	if (!(val>>16)) { r=2; val>>=8; } else { r=0; val>>=24; }
@@ -298,7 +303,7 @@ inline static int LZ4_NbCommonBytes (register U32 val)
     _BitScanForward( &r, val );
     return (int)(r>>3);
     #elif defined(__GNUC__) && (GCC_VERSION >= 304) && !defined(LZ4_FORCE_SW_BITCOUNT)
-    return (__builtin_ctz(val) >> 3); 
+    return (__builtin_ctz(val) >> 3);
     #else
 	static const int DeBruijnBytePos[32] = { 0, 0, 3, 0, 3, 1, 3, 0, 3, 2, 2, 1, 3, 2, 0, 1, 3, 3, 1, 2, 2, 2, 2, 0, 3, 1, 2, 0, 1, 0, 1, 1 };
 	return DeBruijnBytePos[((U32)((val & -(S32)val) * 0x077CB531U)) >> 27];
@@ -309,26 +314,23 @@ inline static int LZ4_NbCommonBytes (register U32 val)
 #endif
 
 
-//****************************
-// Public functions
-//****************************
-
-int LZ4_compressBound(int isize)
-{
-	return (isize + (isize/255) + 16);
-}
-
-
 
 //******************************
 // Compression functions
 //******************************
 
-int LZ4_compressCtx(void** ctx,
+// LZ4_compressCtx :
+// -----------------
+// Compress 'isize' bytes from 'source' into an output buffer 'dest' of maximum size 'maxOutputSize'.
+// If it cannot achieve it, compression will stop, and result of the function will be zero.
+// return : the number of bytes written in buffer 'dest', or 0 if the compression fails
+
+static inline int LZ4_compressCtx(void** ctx,
 				 const char* source,
 				 char* dest,
-				 int isize)
-{	
+				 int isize,
+				 int maxOutputSize)
+{
 #if HEAPMODE
 	struct refTables *srt = (struct refTables *) (*ctx);
 	HTYPE* HashTable;
@@ -336,7 +338,7 @@ int LZ4_compressCtx(void** ctx,
 	HTYPE HashTable[HASHTABLESIZE] = {0};
 #endif
 
-	const BYTE* ip = (BYTE*) source;       
+	const BYTE* ip = (BYTE*) source;
 	INITBASE(base);
 	const BYTE* anchor = ip;
 	const BYTE* const iend = ip + isize;
@@ -344,16 +346,17 @@ int LZ4_compressCtx(void** ctx,
 #define matchlimit (iend - LASTLITERALS)
 
 	BYTE* op = (BYTE*) dest;
-	
+	BYTE* const oend = op + maxOutputSize;
+
 	int len, length;
 	const int skipStrength = SKIPSTRENGTH;
 	U32 forwardH;
 
 
-	// Init 
+	// Init
 	if (isize<MINLENGTH) goto _last_literals;
 #if HEAPMODE
-	if (*ctx == NULL) 
+	if (*ctx == NULL)
 	{
 		srt = (struct refTables *) malloc ( sizeof(struct refTables) );
 		*ctx = (void*) srt;
@@ -368,9 +371,9 @@ int LZ4_compressCtx(void** ctx,
 	// First Byte
 	HashTable[LZ4_HASH_VALUE(ip)] = ip - base;
 	ip++; forwardH = LZ4_HASH_VALUE(ip);
-	
+
 	// Main Loop
-    for ( ; ; ) 
+    for ( ; ; )
 	{
 		int findMatchAttempts = (1U << skipStrength) + 3;
 		const BYTE* forwardIp = ip;
@@ -398,8 +401,28 @@ int LZ4_compressCtx(void** ctx,
 		// Encode Literal length
 		length = ip - anchor;
 		token = op++;
-		if (length>=(int)RUN_MASK) { *token=(RUN_MASK<<ML_BITS); len = length-RUN_MASK; for(; len > 254 ; len-=255) *op++ = 255; *op++ = (BYTE)len; } 
+		if unlikely(op + length + (2 + 1 + LASTLITERALS) + (length>>8) >= oend) return 0; 		// Check output limit
+#ifdef _MSC_VER
+		if (length>=(int)RUN_MASK) 
+		{ 
+			int len = length-RUN_MASK; 
+			*token=(RUN_MASK<<ML_BITS); 
+			if (len>254)
+			{
+				do { *op++ = 255; len -= 255; } while (len>254);
+				*op++ = (BYTE)len; 
+				memcpy(op, anchor, length);
+				op += length;
+				goto _next_match;
+			}
+			else
+			*op++ = (BYTE)len; 
+		}
 		else *token = (length<<ML_BITS);
+#else
+		if (length>=(int)RUN_MASK) { *token=(RUN_MASK<<ML_BITS); len = length-RUN_MASK; for(; len > 254 ; len-=255) *op++ = 255; *op++ = (BYTE)len; }
+		else *token = (length<<ML_BITS);
+#endif
 
 		// Copy Literals
 		LZ4_BLINDCOPY(anchor, op, length);
@@ -426,7 +449,7 @@ _endCount:
 		// Encode MatchLength
 		len = (ip - anchor);
 		if (len>=(int)ML_MASK) { *token+=ML_MASK; len-=ML_MASK; for(; len > 509 ; len-=510) { *op++ = 255; *op++ = 255; } if (len > 254) { len-=255; *op++ = 255; } *op++ = (BYTE)len; }
-		else *token += len;	
+		else *token += len;
 
 		// Test end of chunk
 		if (ip > mflimit) { anchor = ip;  break; }
@@ -440,7 +463,7 @@ _endCount:
 		if ((ref > ip - (MAX_DISTANCE + 1)) && (A32(ref) == A32(ip))) { token = op++; *token=0; goto _next_match; }
 
 		// Prepare next loop
-		anchor = ip++; 
+		anchor = ip++;
 		forwardH = LZ4_HASH_VALUE(ip);
 	}
 
@@ -448,7 +471,7 @@ _last_literals:
 	// Encode Last Literals
 	{
 		int lastRun = iend - anchor;
-		if ((LZ4_COMPRESSMIN>0) && (((op - (BYTE*)dest) + lastRun + 1 + ((lastRun-15)/255)) > isize - LZ4_COMPRESSMIN)) return 0;
+		if (((char*)op - dest) + lastRun + 1 + ((lastRun-15)/255) >= maxOutputSize) return 0;
 		if (lastRun>=(int)RUN_MASK) { *op++=(RUN_MASK<<ML_BITS); lastRun-=RUN_MASK; for(; lastRun > 254 ; lastRun-=255) *op++ = 255; *op++ = (BYTE) lastRun; }
 		else *op++ = (lastRun<<ML_BITS);
 		memcpy(op, anchor, iend - anchor);
@@ -467,10 +490,11 @@ _last_literals:
 #define HASH64KTABLESIZE (1U<<HASHLOG64K)
 #define LZ4_HASH64K_FUNCTION(i)	(((i) * 2654435761U) >> ((MINMATCH*8)-HASHLOG64K))
 #define LZ4_HASH64K_VALUE(p)	LZ4_HASH64K_FUNCTION(A32(p))
-int LZ4_compress64kCtx(void** ctx,
+static inline int LZ4_compress64kCtx(void** ctx,
 				 const char* source,
 				 char* dest,
-				 int isize)
+				 int isize,
+				 int maxOutputSize)
 {
 #if HEAPMODE
 	struct refTables *srt = (struct refTables *) (*ctx);
@@ -487,16 +511,17 @@ int LZ4_compress64kCtx(void** ctx,
 #define matchlimit (iend - LASTLITERALS)
 
 	BYTE* op = (BYTE*) dest;
+	BYTE* const oend = op + maxOutputSize;
 
 	int len, length;
 	const int skipStrength = SKIPSTRENGTH;
 	U32 forwardH;
 
 
-	// Init 
+	// Init
 	if (isize<MINLENGTH) goto _last_literals;
 #if HEAPMODE
-	if (*ctx == NULL) 
+	if (*ctx == NULL)
 	{
 		srt = (struct refTables *) malloc ( sizeof(struct refTables) );
 		*ctx = (void*) srt;
@@ -540,8 +565,28 @@ int LZ4_compress64kCtx(void** ctx,
 		// Encode Literal length
 		length = ip - anchor;
 		token = op++;
+		if unlikely(op + length + (2 + 1 + LASTLITERALS) + (length>>8) >= oend) return 0; 		// Check output limit
+#ifdef _MSC_VER
+		if (length>=(int)RUN_MASK) 
+		{ 
+			int len = length-RUN_MASK; 
+			*token=(RUN_MASK<<ML_BITS); 
+			if (len>254)
+			{
+				do { *op++ = 255; len -= 255; } while (len>254);
+				*op++ = (BYTE)len; 
+				memcpy(op, anchor, length);
+				op += length;
+				goto _next_match;
+			}
+			else
+			*op++ = (BYTE)len; 
+		}
+		else *token = (length<<ML_BITS);
+#else
 		if (length>=(int)RUN_MASK) { *token=(RUN_MASK<<ML_BITS); len = length-RUN_MASK; for(; len > 254 ; len-=255) *op++ = 255; *op++ = (BYTE)len; }
 		else *token = (length<<ML_BITS);
+#endif
 
 		// Copy Literals
 		LZ4_BLINDCOPY(anchor, op, length);
@@ -568,7 +613,7 @@ _endCount:
 		// Encode MatchLength
 		len = (ip - anchor);
 		if (len>=(int)ML_MASK) { *token+=ML_MASK; len-=ML_MASK; for(; len > 509 ; len-=510) { *op++ = 255; *op++ = 255; } if (len > 254) { len-=255; *op++ = 255; } *op++ = (BYTE)len; }
-		else *token += len;	
+		else *token += len;
 
 		// Test end of chunk
 		if (ip > mflimit) { anchor = ip;  break; }
@@ -590,38 +635,46 @@ _last_literals:
 	// Encode Last Literals
 	{
 		int lastRun = iend - anchor;
-		if ((LZ4_COMPRESSMIN>0) && (((op - (BYTE*)dest) + lastRun + 1 + ((lastRun-15)/255)) > isize - LZ4_COMPRESSMIN)) return 0;
+		if (((char*)op - dest) + lastRun + 1 + ((lastRun)>>8) >= maxOutputSize) return 0;
 		if (lastRun>=(int)RUN_MASK) { *op++=(RUN_MASK<<ML_BITS); lastRun-=RUN_MASK; for(; lastRun > 254 ; lastRun-=255) *op++ = 255; *op++ = (BYTE) lastRun; }
 		else *op++ = (lastRun<<ML_BITS);
 		memcpy(op, anchor, iend - anchor);
 		op += iend-anchor;
-	} 
+	}
 
 	// End
 	return (int) (((char*)op)-dest);
 }
 
 
-
-int LZ4_compress(const char* source,
-				 char* dest,
-				 int isize)
+int LZ4_compress_limitedOutput(const char* source, 
+							   char* dest, 
+							   int isize, 
+							   int maxOutputSize)
 {
 #if HEAPMODE
 	void* ctx = malloc(sizeof(struct refTables));
 	int result;
 	if (isize < LZ4_64KLIMIT)
-		result = LZ4_compress64kCtx(&ctx, source, dest, isize);
-	else result = LZ4_compressCtx(&ctx, source, dest, isize);
+		result = LZ4_compress64kCtx(&ctx, source, dest, isize, maxOutputSize);
+	else result = LZ4_compressCtx(&ctx, source, dest, isize, maxOutputSize);
 	free(ctx);
 	return result;
 #else
-	if (isize < (int)LZ4_64KLIMIT) return LZ4_compress64kCtx(NULL, source, dest, isize);
-	return LZ4_compressCtx(NULL, source, dest, isize);
+	if (isize < (int)LZ4_64KLIMIT) return LZ4_compress64kCtx(NULL, source, dest, isize, maxOutputSize);
+	return LZ4_compressCtx(NULL, source, dest, isize, maxOutputSize);
 #endif
 }
 
 
+int LZ4_compress(const char* source,
+				 char* dest,
+				 int isize)
+{
+	return LZ4_compress_limitedOutput(source, dest, isize, LZ4_compressBound(isize));
+}
+
+
 
 
 //****************************
@@ -637,7 +690,7 @@ int LZ4_compress(const char* source,
 int LZ4_uncompress(const char* source,
 				 char* dest,
 				 int osize)
-{	
+{
 	// Local Variables
 	const BYTE* restrict ip = (const BYTE*) source;
 	const BYTE* restrict ref;
@@ -663,7 +716,7 @@ int LZ4_uncompress(const char* source,
 		cpy = op+length;
 		if unlikely(cpy>oend-COPYLENGTH)
 		{
-			if (cpy > oend) goto _output_error;
+			if (cpy > oend) goto _output_error;          // Error : request to write beyond destination buffer
 			memcpy(op, ip, length);
 			ip += length;
 			break;    // Necessarily EOF
@@ -672,7 +725,7 @@ int LZ4_uncompress(const char* source,
 
 		// get offset
 		LZ4_READ_LITTLEENDIAN_16(ref,cpy,ip); ip+=2;
-		if (ref < (BYTE* const)dest) goto _output_error;		
+		if (ref < (BYTE* const)dest) goto _output_error;   // Error : offset create reference outside destination buffer
 
 		// get matchlength
 		if ((length=(token&ML_MASK)) == ML_MASK) { for (;*ip==255;length+=255) {ip++;} length += *ip++; }
@@ -697,7 +750,7 @@ int LZ4_uncompress(const char* source,
 		cpy = op + length - (STEPSIZE-4);
 		if (cpy>oend-COPYLENGTH)
 		{
-			if (cpy > oend) goto _output_error;	
+			if (cpy > oend) goto _output_error;             // Error : request to write beyond destination buffer
 			LZ4_SECURECOPY(ref, op, (oend-COPYLENGTH));
 			while(op<cpy) *op++=*ref++;
 			op=cpy;
@@ -722,7 +775,7 @@ int LZ4_uncompress_unknownOutputSize(
 				char* dest,
 				int isize,
 				int maxOutputSize)
-{	
+{
 	// Local Variables
 	const BYTE* restrict ip = (const BYTE*) source;
 	const BYTE* const iend = ip + isize;
@@ -749,19 +802,19 @@ int LZ4_uncompress_unknownOutputSize(
 		cpy = op+length;
 		if ((cpy>oend-COPYLENGTH) || (ip+length>iend-COPYLENGTH))
 		{
-			if (cpy > oend) goto _output_error;
-			if (ip+length > iend) goto _output_error;
+			if (cpy > oend) goto _output_error;          // Error : request to write beyond destination buffer
+			if (ip+length > iend) goto _output_error;    // Error : request to read beyond source buffer
 			memcpy(op, ip, length);
 			op += length;
 			ip += length;
-			if (ip<iend) goto _output_error;
+			if (ip<iend) goto _output_error;             // Error : LZ4 format violation
 			break;    // Necessarily EOF, due to parsing restrictions
 		}
 		LZ4_WILDCOPY(ip, op, cpy); ip -= (op-cpy); op = cpy;
 
 		// get offset
 		LZ4_READ_LITTLEENDIAN_16(ref,cpy,ip); ip+=2;
-		if (ref < (BYTE* const)dest) goto _output_error;
+		if (ref < (BYTE* const)dest) goto _output_error;   // Error : offset creates reference outside of destination buffer
 
 		// get matchlength
 		if ((length=(token&ML_MASK)) == ML_MASK) { while (ip<iend) { int s = *ip++; length +=s; if (s==255) continue; break; } }
@@ -786,7 +839,7 @@ int LZ4_uncompress_unknownOutputSize(
 		cpy = op + length - (STEPSIZE-4);
 		if (cpy>oend-COPYLENGTH)
 		{
-			if (cpy > oend) goto _output_error;	
+			if (cpy > oend) goto _output_error;           // Error : request to write outside of destination buffer
 			LZ4_SECURECOPY(ref, op, (oend-COPYLENGTH));
 			while(op<cpy) *op++=*ref++;
 			op=cpy;
